/***************************************************************************
;
;    This module is part of the BiPOM developer image for CloudGate Monnit
;    support. All the information in this source file and other source files
;    in the same developer image are copyrighted property of BiPOM Electronics.
;    The source files and the contained information in the source files are
;    confidential and should not be disclosed to anybody other than BiPOM,
;    Option and Monnit software developers, and only to those that have a need to
;    know.
;
;    BiPOM Electronics disclaims all warranties, express or implied warranties
;    of merchantability and fitness for a particular purpose. In no event shall
;    BiPOM Electronics be liable for any indirect, special, incidental or
;    consequential damages in connection  with or arising from the use of this
;    program.
;
;    Please e-mail any questions and suggestions to tech@bipom.com
;
***************************************************************************/

#ifndef _BIPOM_WRTU_LOGGER_
#define _BIPOM_WRTU_LOGGER_

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h> 

#ifdef __cplusplus
extern "C"{
#endif 

	/***************************************************************************/
	#define WRTU_LOG_DIRPATH_LEN			128
	#define WRTU_LOG_MESSAGE_BUF_SIZE		(5 * 1024)
	#define WRTU_LOG_TIME_BUF_SIZE			32
	#define WRTU_LOG_TYPE_BUF_SIZE			16
	#define WRTU_LOG_MAX_LINE_SIZE			(WRTU_LOG_MESSAGE_BUF_SIZE + WRTU_LOG_TIME_BUF_SIZE + WRTU_LOG_TYPE_BUF_SIZE)

	/***************************************************************************/
	#define WRTU_LOG_TYPE_MESSAGE			(1 << 0)
	#define WRTU_LOG_TYPE_WARNING			(1 << 1)
	#define WRTU_LOG_TYPE_ERROR				(1 << 2)
	#define WRTU_LOG_TYPE_DEBUG				(1 << 3)
	#define WRTU_LOG_ALL					(WRTU_LOG_TYPE_ERROR | WRTU_LOG_TYPE_WARNING | WRTU_LOG_TYPE_MESSAGE | WRTU_LOG_TYPE_DEBUG)

	/***************************************************************************/
	typedef struct _wrtu_logger
	{
		char Buffer[WRTU_LOG_MESSAGE_BUF_SIZE];
		char TypeBuffer[WRTU_LOG_TYPE_BUF_SIZE];
		char TimeBuffer[WRTU_LOG_TIME_BUF_SIZE];

		int IsConsoleOutputEnabled;
		int IsFileOutputEnabled;

		int LogLevelMask;

		char DirPath[WRTU_LOG_DIRPATH_LEN];
		char Prefix[WRTU_LOG_DIRPATH_LEN];
		
		int  MaxFileSize;
		int  MaxFileCount;

	} Logger;

	void LOG_Init(const char *pDirName, const char *pPrefix, const int maxFileSize, const int maxFileCount);

	void LOG_LogRecord(int LogRecordType, char * pMsgFormat, va_list vList);

	void LOG_EnableConsoleOutput(int Enabled);
	void LOG_EnableFileOutput(int Enabled);
	void LOG_SetLogLevel(int Mask);

	void LOG_Error(char * pMsgFormat, ...);
	void LOG_Warning(char * pMsgFormat, ...);
	void LOG_Message(char * pMsgFormat, ...);
	void LOG_Debug(char * pMsgFormat, ...);
	
	void LOG_RenameBigFiles();
	void LOG_ParselogLine(const char *pLine, char *pDateTime, char *pType, char *pMsg);

#ifdef __cplusplus
}
#endif

#endif