/***************************************************************************
;
;    This module is part of the BiPOM developer image for CloudGate Monnit
;    support. All the information in this source file and other source files
;    in the same developer image are copyrighted property of BiPOM Electronics.
;    The source files and the contained information in the source files are
;    confidential and should not be disclosed to anybody other than BiPOM,
;    Option and Monnit software developers, and only to those that have a need to
;    know.
;
;    BiPOM Electronics disclaims all warranties, express or implied warranties
;    of merchantability and fitness for a particular purpose. In no event shall
;    BiPOM Electronics be liable for any indirect, special, incidental or
;    consequential damages in connection  with or arising from the use of this
;    program.
;
;    Please e-mail any questions and suggestions to tech@bipom.com
;
***************************************************************************/

#ifndef _BIPOM_MODBUS_
#define _BIPOM_MODBUS_

#include "mb_request_info.h"

#ifdef __cplusplus
extern "C"{
#endif 

	/***************************************************************************/
	#define MB_COILS_REGS_ORIGIN			1
	#define MB_COILS_REGS_LAST				9999
	#define MB_DI_REGS_ORIGIN				10001
	#define MB_DI_REGS_LAST					19999
	#define MB_AI_REGS_ORIGIN				30001
	#define MB_AI_REGS_LAST					39999
	#define MB_HOLDING_REGS_ORIGIN			40001
	#define MB_HOLDING_REGS_LAST			49999

	#define MB_FUNC_READ_COIL				1
	#define MB_FUNC_WRITE_COIL				5
	#define MB_FUNC_READ_DI					2
	#define MB_FUNC_READ_AI					4
	#define MB_FUNC_READ_HR					3
	#define MB_FUNC_WRITE_HR				6
	#define MB_FUNC_WRITE_BATCH_COIL		15
	#define MB_FUNC_WRITE_BATCH_HR			16
	#define MB_FUNC_EXECUTE_CMD				20

	#define MB_REPLY_LEN_READ_COIL			4
	#define MB_REPLY_LEN_WRITE_COIL			6
	#define MB_REPLY_LEN_READ_DI			4
	#define MB_REPLY_LEN_READ_AI			5
	#define MB_REPLY_LEN_READ_HR			5
	#define MB_REPLY_LEN_WRITE_HR			6
	#define MB_REPLY_LEN_EXCEPTION			3
	#define MB_REPLY_LEN_WRITE_BATCH_COIL	6
	#define MB_REPLY_LEN_WRITE_BATCH_HR		6

	#define MB_RTU_CRC16_LEN				2

	/***************************************************************************/
	#define MB_PROTO_CMD_READ_CFG           	0x01
	#define MB_PROTO_CMD_WRITE_CFG          	0x02

	#define MB_PROTO_CMD_READ_TIME          	0x03
	#define MB_PROTO_CMD_WRITE_TIME         	0x04

	#define MB_PROTO_CMD_READ_TAGS_LIST     	0x05
	#define MB_PROTO_CMD_READ_TAG           	0x06
	#define MB_PROTO_CMD_WRITE_TAG          	0x07
	#define MB_PROTO_CMD_DELETE_TAG         	0x08

	#define MB_PROTO_CMD_INIT_LOG_READING   	0x09
	#define MB_PROTO_CMD_READ_LOG           	0x0A
	#define MB_PROTO_CMD_CLEAR_LOG          	0x0B

	#define MB_PROTO_CMD_SET_DEFAULTS       	0x0C
	#define MB_PROTO_CMD_RESTART            	0x0D

	#define MB_PROTO_CMD_READ_DF_SECTOR     	0x0E
	#define MB_PROTO_CMD_WTITE_DF_SECTOR    	0x0F
	#define MB_PROTO_CMD_READ_DF_CRC16      	0x10

	#define MB_PROTO_CMD_READ_SD_SECTOR     	0x11
	#define MB_PROTO_CMD_WTITE_SD_SECTOR    	0x12
	#define MB_PROTO_CMD_READ_SD_CRC16      	0x13

	#define MB_PROTO_CMD_READ_ADC_CALIBRATION	0x14
	#define MB_PROTO_CMD_CALIBRATE_ADC			0x15

	#define MB_PROTO_CMD_DELETE_ALL_TAGS		0x16

	#define MB_PROTO_CMD_SET_LOGGER_STATE		0x17
	#define MB_PROTO_CMD_GET_LOGGER_STATE		0x18

	#define MB_PROTO_CMD_SET_FEATURES			0x19
	#define MB_PROTO_CMD_GET_FEATURES			0x1A

	#define MB_PROTO_CMD_READ_RS485_CFG			0x1B
	#define MB_PROTO_CMD_WRITE_RS485_CFG		0x1C

	#define MB_PROTO_CMD_READ_HW_STATUS			0x1D

	#define MB_PROTO_CMD_READ_CONTACTS_LIST		0x1E
	#define MB_PROTO_CMD_READ_CONTACT			0x1F
	#define MB_PROTO_CMD_WRITE_CONTACT			0x20
	#define MB_PROTO_CMD_DELETE_CONTACT			0x21
	#define MB_PROTO_CMD_DELETE_ALL_CONTACTS	0x22

	#define MB_PROTO_CMD_READ_MESSAGES_LIST		0x23
	#define MB_PROTO_CMD_READ_MESSAGE			0x24
	#define MB_PROTO_CMD_WRITE_MESSAGE			0x25
	#define MB_PROTO_CMD_DELETE_MESSAGE			0x26
	#define MB_PROTO_CMD_DELETE_ALL_MESSAGES	0x27

	#define MB_PROTO_CMD_READ_UART1_CFG			0x28
	#define MB_PROTO_CMD_WRITE_UART1_CFG		0x29

	#define MB_PROTO_CMD_READ_APN_CFG			0x2A
	#define MB_PROTO_CMD_WRITE_APN_CFG			0x2B

	#define MB_PROTO_CMD_GET_TLC2543_SELFTEST	0x2C

	#define MB_PROTO_CMD_READ_VMAP_LIST			0x2D
	#define MB_PROTO_CMD_READ_VMAP_ITEM			0x2E
	#define MB_PROTO_CMD_WRITE_VMAP_ITEM		0x2F
	#define MB_PROTO_CMD_DELETE_VMAP_ITEM		0x30
	#define MB_PROTO_CMD_DELETE_ALL_VMAP_ITEMS	0x31


	#define MB_PROTO_CMD_READ_DATA_PUSHING_CFG		0x32
	#define MB_PROTO_CMD_WRITE_DATA_PUSHING_CFG		0x33

	#define MB_PROTO_CMD_SEND_DATA_PUSHING_START	0x34
	#define MB_PROTO_CMD_SEND_DATA_PUSHING_EOF		0x35
	#define MB_PROTO_CMD_SEND_DATA_PUSHING_RECORDS	0x36

	#define MB_PROTO_CMD_READ_SMTP_CFG				0x37
	#define MB_PROTO_CMD_WRITE_SMTP_CFG				0x38

	#define MB_PROTO_CMD_READ_TAG_VALUE				0x39
	#define MB_PROTO_CMD_READ_ALL_TAG_VALUES		0x40
	#define MB_PROTO_CMD_READ_ALL_TAG_VALUES_V2		0x45

	#define MB_PROTO_CMD_READ_CFG_CHANGED_ID		0x41

	#define MB_PROTO_CMD_READ_PENDING_ALARMS		0x46
	#define MB_PROTO_CMD_START_DATA_PUSH			0x3A
	#define MB_PROTO_CMD_READ_DATA_PUSH_DATA		0x3B

	#define MB_PROTO_CMD_READ_DO_STATE_CONFIG		0x3C
	#define MB_PROTO_CMD_WRITE_DO_STATE_CONFIG		0x3D

	#define MB_PROTO_CMD_SAVE_COUNTERS				0x3E
	#define MB_PROTO_CMD_LOG_EVENT					0x3F
	#define MB_PROTO_CMD_GET_LOG_INFO				0x47

	#define MB_PROTO_CMD_READ_BMAP_LIST				0x48
	#define MB_PROTO_CMD_READ_BMAP_ITEM				0x49
	#define MB_PROTO_CMD_WRITE_BMAP_ITEM			0x4A
	#define MB_PROTO_CMD_DELETE_BMAP_ITEM			0x4B
	#define MB_PROTO_CMD_DELETE_ALL_BMAP_ITEMS		0x4C
	#define MB_PROTO_CMD_DELETE_TAG_BMAP_ITEMS		0x4D

    #define MB_PROTO_CMD_READ_UEXPR_LIST		    0x4E
    #define MB_PROTO_CMD_READ_UEXPR_ITEM		    0x4F
    #define MB_PROTO_CMD_WRITE_UEXPR_ITEM			0x50
    #define MB_PROTO_CMD_DELETE_UEXPR_ITEM			0x51
    #define MB_PROTO_CMD_DELETE_ALL_UEXPR_ITEMS	    0x52

    #define MB_PROTO_CMD_READ_MODEM_FW_REV			0x53
    #define MB_PROTO_CMD_READ_DATA_PUSH_INFO		0x54
    #define MB_PROTO_CMD_READ_STATISTIC			    0x55

    #define MB_PROTO_CMD_WRITE_DATAPUSH_INFO		0x56

	#define MB_PROTO_CMD_GET_STATUS					0xFF
	
	
	/***************************************************************************/
	#define WRTU_BOARD_RTU						0
	#define MB_PROTO_CG_RTU						0x5A
	#define MB_PROTO_CMD_POSITION			 	4
	#define E_DS_FIND_EOF						1013	

	/***************************************************************************/
	#define MB_PROTO_STATE_IDLE					0
	#define MB_PROTO_STATE_RECEIVE				1
	#define MB_PROTO_STATE_PROCESS				2

	/***************************************************************************/
	#define MB_RTU_HEADER_LENGTH				8
	#define MB_RTU_BYTE_COUNT_LOCATION			6
	#define MB_RTU_RTU_LOCATION					0	
	#define MB_RTU_FUNC_LOCATION				1
	#define MB_RTU_LEN_LOCATION					2
	#define MB_EXCEPTION_LEN					5
	#define MB_EXCEPTION_CODE_POS				2

	/***************************************************************************/
	#define MB_TCP_HEADER_LENGTH				6
	#define MB_TCP_MBDATA_LENGTH				6
	#define MB_TCP_REQUEST_LENGTH				12
	#define MB_TCP_REPLY_LENGTH					11
	#define MB_TCP_EXCP_REPLY_LENGTH			9

	#define MB_TCP_PROTO_ID_LOCATION_1			2
	#define MB_TCP_PROTO_ID_LOCATION_2			3
	#define MB_TCP_MSG_LEN_HI_LOCATION			4
	#define MB_TCP_MSG_LEN_LO_LOCATION			5
	#define MB_TCP_ID_LOCATION					6
	#define MB_TCP_FUNC_LOCATION				7
		
	/***************************************************************************/
	#define MB_FUNC_READ_COIL_STATUS			1
	#define MB_FUNC_READ_DI_STATUS				2
	#define MB_FUNC_READ_HREG					3
	#define MB_FUNC_READ_AI_REG					4
	#define MB_FUNC_SET_COIL_STATUS				5
	#define MB_FUNC_SET_HREG					6
	#define MB_FUNC_SET_MUL_COILS				15
	#define MB_FUNC_SET_MUL_HREG				16
	#define MB_FUNC_EXECUTE_CMD 				20

	/***************************************************************************/
	#define MB_REPLY_LEN_READ_COIL 				4
	#define MB_REPLY_LEN_WRITE_COIL 			6
	#define MB_REPLY_LEN_READ_DI 				4
	#define MB_REPLY_LEN_READ_AI 				5
	#define MB_REPLY_LEN_READ_HR 				5
	#define MB_REPLY_LEN_WRITE_HR 				6
	#define MB_REPLY_LEN_EXCEPTION 				3
	#define MB_REPLY_LEN_WRITE_BATCH_COIL 		6
	#define MB_REPLY_LEN_WRITE_BATCH_HR 		6

	/***************************************************************************/
	#define MB_COILS_REGS_ORIGIN				1
	#define MB_COILS_REGS_LAST					9999
	#define MB_DI_REGS_ORIGIN					10001
	#define MB_DI_REGS_LAST						19999
	#define MB_AI_REGS_ORIGIN					30001
	#define MB_AI_REGS_LAST						39999
	#define MB_HOLDING_REGS_ORIGIN				40001
	#define MB_HOLDING_REGS_LAST				49999

	/***************************************************************************/
	#define MB_EXCP_ILLEGAL_FUNC				1
	#define MB_EXCP_ILLEGAL_ADDR				2
	#define MB_EXCP_ILLEGAL_VALUE				3
	#define MB_EXCP_FAILURE						4
	#define MB_EXCP_ACK							5
	#define MB_EXCP_BUSY						6
	#define MB_EXCP_NOT_ACK						7
	#define MB_EXCP_NO_GATEWAY					10
		
	/***************************************************************************/
	#define MB_ON								0xFF00
	#define MB_OFF								0x0000


	/***************************************************************************/
	/*   MODBUS I/O REGISTERS   */
	/***************************************************************************/
	#define WRTU_MB_FIRST_COIL_REG			1
	#define WRTU_MB_LAST_COIL_REG			2
	#define WRTU_MB_FIRST_INPUT_REG			10001
	#define WRTU_MB_LAST_INPUT_REG			10004

	/***************************************************************************/
	/*   MODBUS ADC REGISTERS   */
	/***************************************************************************/
	#define WRTU_MB_FIRST_ADC_REG			30001
	#define WRTU_MB_LAST_ADC_REG			30005
	#define WRTU_MB_FIRST_ADC_RMS_REG		30006
	#define WRTU_MB_LAST_ADC_RMS_REG		30007
	#define WRTU_MB_FIRST_TLC2543_REG		30008
	#define WRTU_MB_LAST_TLC2543_REG		30018

	/***************************************************************************/
	/*   MODBUS HOLDING REGISTERS   */
	/***************************************************************************/
	#define WRTU_MB_HW_TYPE_REG				40001
	#define WRTU_MB_RTU_REG					40100
	#define WRTU_MB_YEAR					40101
	#define WRTU_MB_MONTH					40102
	#define WRTU_MB_DAY						40103
	#define WRTU_MB_HOUR					40104
	#define WRTU_MB_MIN						40105
	#define WRTU_MB_SEC						40106
	#define WRTU_MB_REBOOT_SRC				40107
	#define WRTU_MB_LAST_ERR				40108
	#define WRTU_MB_UPTIME_LOW				40109
	#define WRTU_MB_UPTIME_HIGH				40110
	#define WRTU_MB_HARDWARE_STATUS			40111
	#define WRTU_MB_FIRMWARE_VER			40112
	#define WRTU_MB_DI_FREQ_FIRST			40113
	#define WRTU_MB_DI_FREQ_SECOND			40114
	#define WRTU_MB_MB_TOTAL_ERRORS			40115
	#define WRTU_MB_MB_TOTAL_EXCEPTIONS		40116
	#define WRTU_MB_TOTAL_DF_WRITE_ERRORS	40117
	#define WRTU_MB_TOTAL_DF_READ_ERRORS	40118
	#define WRTU_MB_TOTAL_SD_WRITE_ERRORS	40119
	#define WRTU_MB_TOTAL_SD_READ_ERRORS	40120
	#define WRTU_MB_MODEM_TYPE				40121
	#define WRTU_MB_DI_CNT_CHANNEL1_HI		40122
	#define WRTU_MB_DI_CNT_CHANNEL1_LO		40123
	#define WRTU_MB_DI_CNT_CHANNEL2_HI		40124
	#define WRTU_MB_DI_CNT_CHANNEL2_LO		40125
	#define WRTU_MB_DI_CNT_CHANNEL3_HI		40126
	#define WRTU_MB_DI_CNT_CHANNEL3_LO		40127
	#define WRTU_MB_DI_CNT_CHANNEL4_HI		40128
	#define WRTU_MB_DI_CNT_CHANNEL4_LO		40129

	#define WRTU_MB_LOOP_POWER				40130

	#define WRTU_MB_CELL_ENABLED			40131
	#define WRTU_MB_MODEM_DETECTED			40132
	#define WRTU_MB_CSQ_RAW					40133
	#define WRTU_MB_CSQ_DBM					40134
	#define WRTU_MB_SIM_ERROR				40135
	#define WRTU_MB_SIM_STATUS				40136
	#define WRTU_MB_NET_STATUS				40137
	#define WRTU_MB_REG_STATUS				40138

	#define WRTU_MB_SD_CARD_DETECTED		40139
	#define WRTU_MB_SD_CARD_TYPE			40140
	#define WRTU_MB_SD_CARD_BLOCK_SIZE		40141
	#define WRTU_MB_SD_CARD_CAPACITY_LO		40142
	#define WRTU_MB_SD_CARD_CAPACITY_HI		40143

	#define MB_MAX_REQUEST_LEN				512
	#define MB_MAX_RESPONSE_LEN				512
	#define MB_MAX_VALUES_PER_REQUEST		128
		

    #define WRTU_DATAPUSHSTATUS_VER         0x01

	// ****************************************************************************
	typedef struct _mb_tcp_hdr
	{
		int 	Id;
		int 	ProtoId;
		int 	DataLen;

	} ModbusTcpHeader;

	
	/***************************************************************************/
	unsigned int MB_CRC(char *pData, int length);
	
	int MB_BuildException(char *pBuf, int *pBufLen, int excpCode, char rtu, char function);
	int MB_BuildReadRequestData(ModbusRequestInfo *pModbusRequestInfo, int rtu, int regAddr, int regCount);
	int MB_BuildWriteRequestData(ModbusRequestInfo *pModbusRequestInfo, int rtu, int regAddr, int *pValues, int valuesCount);
	
	int MB_IsRtuException(char *pData, int length, char function);
	int MB_CheckRtuResponse(ModbusRequestInfo *pModbusRequestInfo);
	int MB_ParseRtuResponse(ModbusRequestInfo *pModbusRequestInfo);

	int MB_GetRawRequestInfo(char *pRawRequest, int *pFunc, unsigned int *pAddr, int *pValuesCount);
	int MB_GetReplyLengthForRawRequest(char *pRawRequest, int *pReplyLength);
		
	int MB_ParseTcpHeader(char *pData, ModbusTcpHeader *pTcpHdr);
	
#ifdef __cplusplus
}
#endif

#endif