/***************************************************************************
;
;    This module is part of the BiPOM developer image for CloudGate Monnit
;    support. All the information in this source file and other source files
;    in the same developer image are copyrighted property of BiPOM Electronics.
;    The source files and the contained information in the source files are
;    confidential and should not be disclosed to anybody other than BiPOM,
;    Option and Monnit software developers, and only to those that have a need to
;    know.
;
;    BiPOM Electronics disclaims all warranties, express or implied warranties
;    of merchantability and fitness for a particular purpose. In no event shall
;    BiPOM Electronics be liable for any indirect, special, incidental or
;    consequential damages in connection  with or arising from the use of this
;    program.
;
;    Please e-mail any questions and suggestions to tech@bipom.com
;
***************************************************************************/

#ifndef _BIPOM_THREAD_
#define _BIPOM_THREAD_

#include <pthread.h>

#ifdef __cplusplus
extern "C"{
#endif 

	/***************************************************************************/
	#define THREAD_STATE_NOT_STARTED	0
	#define THREAD_STATE_STARTED		1
	#define THREAD_STATE_CANCELLED		2
	#define THREAD_STATE_STOPPED		3
	#define THREAD_STATE_FAILED			4

	struct _thread;

	/***************************************************************************/
	/* define type of thread function */
	typedef int(*pThreadFunc)(int *pStopRequest, void *pParams);
	typedef int(*pThreadCompletedFunc)(struct _thread *pThread, void *pParams);

	typedef struct _thread
	{
		int				IsInit;
		pthread_t		ThreadId;
		pThreadFunc		ThreadFunc;
		void			*pParams;

		int				State;
		int				StopRequest;

		pThreadCompletedFunc	ThreadCompletedFunc;
		void					*pCompletedParams;

	} Thread;

	int THRD_Init(Thread *pThread);
	int THRD_Reset(Thread *pThread);
	int THRD_Start(Thread *pThread, pThreadFunc pFunc, void *pParams);
	int THRD_Stop(Thread *pThread);
	int THRD_Cancel(Thread *pThread);

	int THRD_GetState(Thread *pThread, int *pState);
	int THRD_GetId(Thread *pThread, pthread_t *pThreadId);

	int THRD_SetOnCompleted(Thread *pThread, pThreadCompletedFunc pFunc, void *pParams);

	void * THRD_WorkerThread(void *pParams);
	
#ifdef __cplusplus
}
#endif

#endif