/***************************************************************************
;
;    This module is part of the BiPOM developer image for CloudGate Monnit
;    support. All the information in this source file and other source files
;    in the same developer image are copyrighted property of BiPOM Electronics.
;    The source files and the contained information in the source files are
;    confidential and should not be disclosed to anybody other than BiPOM,
;    Option and Monnit software developers, and only to those that have a need to
;    know.
;
;    BiPOM Electronics disclaims all warranties, express or implied warranties
;    of merchantability and fitness for a particular purpose. In no event shall
;    BiPOM Electronics be liable for any indirect, special, incidental or
;    consequential damages in connection  with or arising from the use of this
;    program.
;
;    Please e-mail any questions and suggestions to tech@bipom.com
;
***************************************************************************/

#ifndef _BIPOM_WRTU_
#define _BIPOM_WRTU_

#include "wrtu_configuration.h"
#include "wrtu_device_info.h"
#include "wrtu_tag_info.h"
#include "wrtu_hardware_info.h"
#include "wrtu_contact_info.h"
#include "wrtu_message_info.h"
#include "wrtu_smtp_info.h"
#include "wrtu_time.h"
#include "wrtu_alarm_info.h"
#include "wrtu_event_info.h"
#include "wrtu_vmap_item.h"
#include "wrtu_bmap_item.h"


#ifdef __cplusplus
extern "C"{
#endif 

    #define WRTU_RTC_INFO_VER			0x01

	/***************************************************************************/
	// BiPOM WRTU Library for CloudGate

	int WRTU_Init(WrtuConfiguration *pCfg);
	int WRTU_Start();
	int WRTU_Stop();
	int WRTU_IsConnected();
	int WRTU_IsSerialPortOpened();

	int WRTU_GetDOCount();
	int WRTU_GetDICount();
	int WRTU_GetAICount();

	int WRTU_ProccessRawModbusRtuRequest(char *pRawRequestData, int requestLength, char *pRawResponse, int *pResponseLength);
	int WRTU_ReadFirmwareVersion(char *pVersion, int maxLength, int *pVersionNum);
	int WRTU_ReadHardwareType(int *pHardwareType);
	int WRTU_ReadDeviceInfo(WrtuDeviceInfo *pDeviceInfo);
	int WRTU_ReadTags(WrtuTagInfo *pTags, int *pTagsCount);
	int WRTU_ReadDataPushCfg(DataPushConfig *pDataPushCfg);
	int WRTU_ReadLoggerState(int *pState);
	int WRTU_ReadRtc(struct tm *pRtcTime);
    int WRTU_WriteRtc(struct tm *pRtcTime);
	int WRTU_ReadUptime(long *pUptime);
	int WRTU_ReadContactCount(int *pContactCount);
	int WRTU_ReadContacts(WrtuContactInfo *pContacts, int *pContactCount);
	int WRTU_ReadMessageCount(int *pMessageCount);
	int WRTU_ReadMessages(WrtuMessageInfo *pMesages, int *pMessageCount);
	int WRTU_ReadVMapItemCount(int *pCount);
	int WRTU_ReadVMapItem(WrtuVMapItem *pVMapItems, int *pCount);
	int WRTU_ReadBMapItemCount(int *pCount);
	int WRTU_ReadBMapItem(WrtuBMapItem *pBMapItems, int *pCount);
	int WRTU_ReadSmtpInfo(WrtuSmtpInfo *pSmtpInfo);

	int WRTU_ReadDO(int channel, int *pState);
	int WRTU_WriteDO(int channel, int state);
	int WRTU_ReadDI(int channel, int *pState);
	int WRTU_ReadAI(int channel, int *pValue);
	int WRTU_ReadTagValue(int id, int *pRawValue, float *pCalcValue);

	int WRTU_ReadAllDO(int *pState, int *pCount);
	int WRTU_WriteAllDO(int *pState, int count);
	int WRTU_ReadAllDI(int *pState, int *pCount);
	int WRTU_ReadAllAI(int *pValue, int *pCount);
	int WRTU_ReadAllTagValues(int *pId, int *pIsValid, int *pRawValue, float *pCalcValue, int *pCount);

	int WRTU_GetLoopPowerState(int *pState);
	int WRTU_SetLoopPowerState(int state);

	int WRTU_LogEvent(int type, int id, int data);

	int WRTU_ReadSDCardInfo(int *pDetected, int *pType, unsigned long long *pSize);

    int WRTU_WriteDataPushInfo(time_t time, int err);

#ifdef __cplusplus
}
#endif

#endif