/***************************************************************************
;
;    This module is part of the BiPOM developer image for CloudGate Monnit
;    support. All the information in this source file and other source files
;    in the same developer image are copyrighted property of BiPOM Electronics.
;    The source files and the contained information in the source files are
;    confidential and should not be disclosed to anybody other than BiPOM,
;    Option and Monnit software developers, and only to those that have a need to
;    know.
;
;    BiPOM Electronics disclaims all warranties, express or implied warranties
;    of merchantability and fitness for a particular purpose. In no event shall
;    BiPOM Electronics be liable for any indirect, special, incidental or
;    consequential damages in connection  with or arising from the use of this
;    program.
;
;    Please e-mail any questions and suggestions to tech@bipom.com
;
***************************************************************************/

#ifndef _BIPOM_WRTU_CONFIGURATION_
#define _BIPOM_WRTU_CONFIGURATION_

#ifdef __cplusplus
extern "C"{
#endif 

	//**************************************************************************
	#define WRTU_REC_DATA			0
	#define WRTU_REC_EVENT			1
	#define WRTU_REC_ALARM			2
	#define WRTU_REC_COUNT			3

	//**************************************************************************
	#define WRTU_CFGID_NOTHING					0
	#define WRTU_CFGID_RESET					(1 << 0)
	#define WRTU_CFGID_DEVICE_INFO				(1 << 1)
	#define WRTU_CFGID_RTC_TIME					(1 << 2)
	#define WRTU_CFGID_TAGS						(1 << 3)
	#define WRTU_CFGID_LOGGER_STATE				(1 << 4)
	#define WRTU_CFGID_FEATURES					(1 << 5)
	#define WRTU_CFGID_RS485					(1 << 6)
	#define WRTU_CFGID_RS232					(1 << 7)
	#define WRTU_CFGID_CONTACTS					(1 << 8)
	#define WRTU_CFGID_MESSAGES					(1 << 9)
	#define WRTU_CFGID_VMAP						(1 << 10)
	#define WRTU_CFGID_APN						(1 << 11)
	#define WRTU_CFGID_DATA_PUSHING				(1 << 12)
	#define WRTU_CFGID_SMTP						(1 << 13)
	#define WRTU_CFGID_LCD						(1 << 14)
	#define WRTU_CFGID_PERIPH					(1 << 15)
	#define WRTU_CFGID_COUNTERS					(1 << 16)
	#define WRTU_CFGID_BMAP						(1 << 17)

	//**************************************************************************
	#define WRTU_DATA_PUSHING_ADDRESS_LEN		128
	#define WRTU_DATA_PUSHING_LOGIN_LEN			128
	#define WRTU_DATA_PUSHING_PASSWORD_LEN		128

	// *********************************************************
	#define WRTU_DATA_PUSHING_FORMAT_CUSTOM				0
	#define WRTU_DATA_PUSHING_FORMAT_BIPOM				1
	#define WRTU_DATA_PUSHING_FORMAT_NEW_BOUNDARY		2

	// *********************************************************
	#define WRTU_URL_PROTO_UNDEFINED	0
	#define WRTU_URL_PROTO_HTTP			1
	#define WRTU_URL_PROTO_HTTPS		2
	#define WRTU_URL_PROTO_FTP			3


	//**************************************************************************
	typedef struct _wrtu_data_push_config
	{
		int 	IsEnabled;
		int 	Protocol;
		int 	Format;
		int 	Port;
		int 	Period;
		int 	Offset;
		char	Address[WRTU_DATA_PUSHING_ADDRESS_LEN];
		char	Login[WRTU_DATA_PUSHING_LOGIN_LEN];
		char	Password[WRTU_DATA_PUSHING_PASSWORD_LEN];

	} DataPushConfig;

	//**************************************************************************
	typedef struct _wrtu_log_record
	{
		unsigned long	id;

		struct tm		time;
		char  			recType;
		unsigned short	tagId;

		union
		{
			float	rawValue;
			int		intValue; 	/* Id of event */
		} data1;

		union
		{
			float	calcValue;
			int		intValue;	/* error code for event */
		} data2;

		short		data3; 		/* AlarmType for Alarm, EventType for Event */

		char 		crc8;		/* checksum of record */

	} LogRecord;

	//**************************************************************************
	typedef int(*pEventAlarmRecordsFunc)(LogRecord *pLogRecords, int recordsCount);
	typedef int(*pEventDataPushRecordsFunc)(LogRecord *pLogRecords, int recordsCount);
	typedef int(*pEventDataPushStartFunc)(int *pIsReady);
	typedef int(*pEventDataPushEofFunc)();
	typedef int(*pEventCfgChangedFunc)(unsigned long cfgId);

	#define WRTU_MAX_SERIALPORT_LENGTH	64

	//**************************************************************************
	typedef struct _wrtu_configuration
	{
		char	SerialPortName [WRTU_MAX_SERIALPORT_LENGTH];
		int		Baudrate;

		pEventAlarmRecordsFunc		pAlarmRecordsFunc;
		pEventDataPushRecordsFunc	pDataPushRecordsFunc;
		pEventDataPushStartFunc		pDataPushStartFunc;
		pEventDataPushEofFunc		pDataPushEofFunc;
		pEventCfgChangedFunc		pCfgChangedFunc;

	} WrtuConfiguration;


#ifdef __cplusplus
}
#endif

#endif