/***************************************************************************
;
;    This module is part of the BiPOM developer image for CloudGate Monnit
;    support. All the information in this source file and other source files
;    in the same developer image are copyrighted property of BiPOM Electronics.
;    The source files and the contained information in the source files are
;    confidential and should not be disclosed to anybody other than BiPOM,
;    Option and Monnit software developers, and only to those that have a need to
;    know.
;
;    BiPOM Electronics disclaims all warranties, express or implied warranties
;    of merchantability and fitness for a particular purpose. In no event shall
;    BiPOM Electronics be liable for any indirect, special, incidental or
;    consequential damages in connection  with or arising from the use of this
;    program.
;
;    Please e-mail any questions and suggestions to tech@bipom.com
;
***************************************************************************/

#ifndef _BIPOM_WRTU_SERVICE_
#define _BIPOM_WRTU_SERVICE_

#include "wrtu_configuration.h"
#include "wrtu_device_status.h"
#include "mb_request_info.h"
#include "serial_port.h"
#include "thread.h"

#ifdef __cplusplus
extern "C"{
#endif 

	#define SRV_PROTO_SIGNATURE			0xA5

	/***************************************************************************/
	#define WRTU_DS_REC_DATA			0
	#define WRTU_DS_REC_EVENT			1
	#define WRTU_DS_REC_ALARM			2

	/***************************************************************************/
	#define SRV_RX_BUFFER_SIZE			2032
	#define SRV_TX_BUFFER_SIZE			2032
		
	#define WRTU_MAX_LOG_RECORDS		256

	/***************************************************************************/
	typedef struct _wrtu_service
	{
		int			IsInit;

		Thread		ServiceThread;
		SerialPort	SerialPort;

		char	SerialPortName[WRTU_MAX_SERIALPORT_LENGTH];
		int		Baudrate;
		int		ConnectTimeout;
		int		ReadTimeout;
		int		WriteTimeout;

		char	RxBuffer[SRV_RX_BUFFER_SIZE];
		int		RxHead;
		int		RxTail;

		char	TxBuffer[SRV_TX_BUFFER_SIZE];
		int		TxHead;
		int		TxTail;

		ModbusRequestInfo	ModbusRequestInt;
		ModbusRequestInfo	ModbusRequestExt;


		WrtuDeviceStatus	DeviceStatus;

		pEventAlarmRecordsFunc				pAlarmRecordsFunc;
		pEventDataPushRecordsFunc			pDataPushRecordsFunc;
		pEventDataPushEofFunc				pDataPushEofFunc;
		pEventDataPushStartFunc				pDataPushStartFunc;
		pEventCfgChangedFunc				pCfgChangedFunc;

		LogRecord	LogRecords[WRTU_MAX_LOG_RECORDS];

	} WrtuService;

	int SRV_ServiceThread(int *pStopRequest, void *pParams);

	WrtuService * SRV();

	int SRV_Init(WrtuConfiguration *pCfg);
	int SRV_Start();
	int SRV_Stop();
	int SRV_IsConnected();
	int SRV_IsSerialPortOpened();
	int SRV_GetReadTimeout();
	int SRV_SetReadTimeout(int timeout);
	int SRV_GetRxBufferSize();
	int SRV_GetTxBufferSize();
	int SRV_ReadToRxBuffer();
	int SRV_GetFromRxBuffer();
	int SRV_ClearRxBuffer();
	int SRV_WriteToTxBuffer(unsigned char *pData, int Count);
	int SRV_WaitOnReply(int *pStopRequest, int responseLength);
	
	int  SRV_ProcessInternalModbusRequest(int *pStopRequest);
	int  SRV_ProcessExternalModbusRequest(int *pStopRequest);
	int  SRV_ProcessModbusRequest(int *pStopRequest, ModbusRequestInfo *pRequestInfo);
	void SRV_StartModbusRequest(ModbusRequestInfo *pRequestInfo);
	void SRV_FinishModbusRequest(ModbusRequestInfo *pRequestInfo, int ec);

	int SRV_WriteModbusRegisters(int regAddr, int *pValues, int count);
	int SRV_ReadModbusRegisters(int regAddr, int *pValues, int count);
	int SRV_SendRequestAndGetReply(ModbusRequestInfo *pRequestInfo);
	int SRV_SendRawRequestAndGetReply(char *pRawRequestData, int requestLength, char *pRawResponse, int *pResponseLength);
	int SRV_WaitOnModbusRequestCompleted(ModbusRequestInfo *pRequestInfo);

	void SRV_SetEvent_AlarmRecords(pEventAlarmRecordsFunc	pAlarmRecordsFunc);
	void SRV_SetEvent_DataPushRecords(pEventDataPushRecordsFunc pDataPushRecordsFunc);
	void SRV_SetEvent_DataPushStart(pEventDataPushStartFunc pDataPushStartFunc);
	void SRV_SetEvent_DataPushEof(pEventDataPushEofFunc pDataPushEofFunc);
	void SRV_SetEvent_CfgChanged(pEventCfgChangedFunc	pCfgChangedFunc);


	int  SRV_GetStatus(int *pStopRequest);
	void SRV_BuildGetStatusCmd();
	void SRV_ParseGetStatusCmd();

	int SRV_ProcessConfigChangedEvent(int *pStopRequest);
	int SRV_ProcessAlarmsEvent(int *pStopRequest);
	int SRV_ProcessDataEvent(int *pStopRequest);

	void SRV_BuildReadChangedCfgIdCmd();
	void SRV_ParseReadChangedCfgIdCmd(unsigned int *pCfgId);

	void SRV_BuildPushDataStartCmd();
	void SRV_ParsePushDataStartCmd(int *pErrorCode);
	void SRV_BuildReadPushDataCmd();
	void SRV_ParseReadPushDataCmd(LogRecord *pLogRecords, int *pRecordsCount, int *pErrorCode);
	
	void SRV_BuildReadPendingAlarmsCmd();
	void SRV_ParseReadPendingAlarmsCmd(LogRecord *pLogRecords, int *pRecordsCount, int *pErrorCode);

	int  SRV_ParseLogRecord(char *pData, int version, LogRecord *pLogRecord);

#ifdef __cplusplus
}
#endif

#endif