/***************************************************************************
;
;    This module is part of the BiPOM developer image for CloudGate Monnit
;    support. All the information in this source file and other source files
;    in the same developer image are copyrighted property of BiPOM Electronics.
;    The source files and the contained information in the source files are
;    confidential and should not be disclosed to anybody other than BiPOM,
;    Option and Monnit software developers, and only to those that have a need to
;    know.
;
;    BiPOM Electronics disclaims all warranties, express or implied warranties
;    of merchantability and fitness for a particular purpose. In no event shall
;    BiPOM Electronics be liable for any indirect, special, incidental or
;    consequential damages in connection  with or arising from the use of this
;    program.
;
;    Please e-mail any questions and suggestions to tech@bipom.com
;
***************************************************************************/

#ifndef _BIPOM_WRTU_TAG_INFO_
#define _BIPOM_WRTU_TAG_INFO_

#ifdef __cplusplus
extern "C"{
#endif 

	/***************************************************************************/
	#define WRTU_MAX_TAGS_COUNT					62

	/***************************************************************************/
	#define WRTU_TAG_NAME_LEN					24
	#define WRTU_TAG_UNITS_LEN					24

	// *********************************************************
	#define WRTU_EQUATION_RAW					0
	#define WRTU_EQUATION_LINEAR				1
	#define WRTU_EQUATION_1						2

	// *********************************************************
	#define WRTU_TAG_TYPE_AI					0
	#define WRTU_TAG_TYPE_AI_RMS				1
	#define WRTU_TAG_TYPE_DI					2
	#define WRTU_TAG_TYPE_DI_FREQ				3
	#define WRTU_TAG_TYPE_DO					4
	#define WRTU_TAG_TYPE_REG					5
	#define WRTU_TAG_TYPE_MB					6
	#define WRTU_TAG_TYPE_TLC2543				7
	#define WRTU_TAG_TYPE_DI_CNT				8
	#define WRTU_TAG_TYPE_DI_OPTO9				9
	#define WRTU_TAG_TYPE_DI_OPTO8DC			10

	// *********************************************************
	#define WRTU_TAG_MB_TYPE_COIL				0
	#define WRTU_TAG_MB_TYPE_DI					1
	#define WRTU_TAG_MB_TYPE_AI					2
	#define WRTU_TAG_MB_TYPE_HREG				3

	//*********************************************************************************************
	#define WRTU_MAX_CONTACTS_PER_ALARM			5
		
	//*********************************************************************************************
	#define WRTU_ALARM_TYPE_NONE				0
	#define WRTU_ALARM_TYPE_LIMIT				1
	#define WRTU_ALARM_TYPE_VALUECHANGE			2
	#define WRTU_ALARM_TYPE_COUNT				3

	//*********************************************************************************************
	#define WRTU_ALARM_NONE						0
	#define WRTU_ALARM_HIHI						1
	#define WRTU_ALARM_HI						2
	#define WRTU_ALARM_NORMAL					3
	#define WRTU_ALARM_LO						4
	#define WRTU_ALARM_LOLO						5
	#define WRTU_ALARM_VALUE_CHANGED			6
	#define WRTU_ALARM_EXACT_VALUE				7
	#define WRTU_TAG_ALARM_COUNT				8
		
	// *********************************************************
	#define WRTU_TAG_MB_VAL_TYPE_INT			0
	#define WRTU_TAG_MB_VAL_TYPE_UINT			1
	#define WRTU_TAG_MB_VAL_TYPE_LONG			2
	#define WRTU_TAG_MB_VAL_TYPE_ULONG			3
	#define WRTU_TAG_MB_VAL_TYPE_FLOAT			4
		
	// *********************************************************
	#define WRTU_TAG_MB_VAL_BO_DEFAULT			0
	#define WRTU_TAG_MB_VAL_BO_BWS				1
	#define WRTU_TAG_MB_VAL_BO_BS				2
	#define WRTU_TAG_MB_VAL_BO_WS			    3

	#define WRTU_TAG_ALARM_VALS_COUNT			20
		
	// *********************************************************
	typedef struct _wrtu_tag_alarm_info
	{
		char	type;

		unsigned int	timeout;
		float	deadband;
	
		float	lolo;
		float	lo;
		float	hi;
		float	hihi;
	
		unsigned short	contactId[WRTU_TAG_ALARM_COUNT-1][WRTU_MAX_CONTACTS_PER_ALARM];
		unsigned short	messageId[WRTU_TAG_ALARM_COUNT-1];

		char 	loloEnabled;
		char 	loEnabled;
		char 	normalEnabled;
		char 	hiEnabled;
		char 	hihiEnabled;
	
		int		alarmValueCount;
		int		alarmValues[WRTU_TAG_ALARM_VALS_COUNT];

	} WrtuTagAlarmInfo;

	// *********************************************************
	typedef struct _wrtu_scaling_info
	{
		char 			isEnabled;
		unsigned int 	zeroScaleCounts;
		unsigned int 	fullScaleCounts;
		float 			zeroScaleOutput;
		float 			fullScaleOutput;
	
	} WrtuTagScalingInfo;

	// *********************************************************
	typedef struct _wrtu_tag_info
	{
		short			id;
	
		char			type;
		char			IsLogEnabled;
		char			IsVMapEnabled;

		char			name[WRTU_TAG_NAME_LEN];
		char			units1[WRTU_TAG_UNITS_LEN];
		char			units2[WRTU_TAG_UNITS_LEN];

		unsigned int	addr;
		unsigned int	logPeriod;
	
		char			equationType;
		float			a;
		float			b;
	
		char			mbRtu;
		char			mbType;
		unsigned int	mbRegister;

		WrtuTagAlarmInfo		alarmInfo;
		WrtuTagScalingInfo		scalingInfo;

		char			mbValueType;
		char			mbValueByteOrder;

		char			IsBMapEnabled;
		unsigned char	BitMapAlarms;
		
	} WrtuTagInfo;

	#define WRTU_TAG_INFO_SIZE		sizeof(WrtuTagInfo)		
	
#ifdef __cplusplus
}
#endif

#endif	